@@ -97,10 +97,13 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc,v 1.29 2005/01/31 20:40:45 fyodor Exp $ */
+/* $Id: NmapOps.cc 2839 2005-09-07 08:26:45Z fyodor $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
+#ifdef WIN32
+#include "winfix.h"
+#endif
 
 NmapOps o;
 
@@ -178,15 +181,18 @@ void NmapOps::Initialize() {
 # ifdef __amigaos__
     isr00t = 1;
 # else
-    isr00t = !(geteuid());
+    if (getenv("NMAP_PRIVILEGED"))
+      isr00t = 1;
+    else
+      isr00t = !(geteuid());
 # endif // __amigaos__
 #else
   isr00t = 1;
-  winip_init();	/* wrapper for all win32 initialization */
 #endif
   debugging = DEBUGGING;
   verbose = DEBUGGING;
   randomize_hosts = 0;
+  sendpref = PACKET_SEND_NOPREF;
   spoofsource = 0;
   device[0] = '\0';
   interactivemode = 0;
@@ -218,6 +224,8 @@ void NmapOps::Initialize() {
   decoyturn = -1;
   osscan = 0;
   servicescan = 0;
+  override_excludeports = 0;
+  version_intensity = 7;
   pingtype = PINGTYPE_UNKNOWN;
   listscan = pingscan = allowall = ackscan = bouncescan = connectscan = 0;
   rpcscan = nullscan = xmasscan = fragscan = synscan = windowscan = 0;
@@ -230,10 +238,14 @@ void NmapOps::Initialize() {
   pTrace = vTrace = false;
   if (datadir) free(datadir);
   datadir = NULL;
-
+#if WIN32
+  Strncpy(tmpxsl, "nmap.xsl", sizeof(tmpxsl));
+#else
   snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
+#endif
   if (xsl_stylesheet) free(xsl_stylesheet);
   xsl_stylesheet = strdup(tmpxsl);
+  spoof_mac_set = false;
 }
 
 bool NmapOps::TCPScan() {
@@ -259,9 +271,13 @@ bool NmapOps::RawScan() {
 
 
 void NmapOps::ValidateOptions() {
-
+#ifdef WIN32
+	const char *privreq = "that WinPcap version 3.1 or higher and iphlpapi.dll be installed. You seem to be missing one or both of these.  Winpcap is available from http://www.winpcap.org.  iphlpapi.dll comes with Win98 and later operating sytems and NT 4.0 with SP4 or greater.  For previous windows versions, you may be able to take iphlpapi.dll from anotyer system and place it in your system32 dir (e.g. c:\\windows\\system32)";
+#else
+	const char *privreq = "root privileges";
+#endif
   if (pingtype == PINGTYPE_UNKNOWN) {
-    if (isr00t && af() == AF_INET) pingtype = PINGTYPE_TCP|PINGTYPE_TCP_USE_ACK|PINGTYPE_ICMP_PING;
+    if (isr00t && af() == AF_INET) pingtype = DEFAULT_PING_TYPES;
     else pingtype = PINGTYPE_TCP; // if nonr00t or IPv6
     num_ping_ackprobes = 1;
     ping_ackprobes[0] = DEFAULT_TCP_PROBE_PORT;
@@ -321,6 +337,13 @@ void NmapOps::ValidateOptions() {
    fatal("Ping scan is not valid with any other scan types (the other ones all include a ping scan");
  }
 
+ if (sendpref == PACKET_SEND_NOPREF) {
+#ifdef WIN32
+   sendpref = PACKET_SEND_ETH_STRONG;
+#else
+   sendpref = PACKET_SEND_IP_WEAK;
+#endif
+ }
 /* We start with stuff users should not do if they are not root */
   if (!isr00t) {
 
@@ -337,35 +360,19 @@ void NmapOps::ValidateOptions() {
 #endif
     
     if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan) {
-#ifndef WIN32
-      fatal("You requested a scan type which requires r00t privileges, and you do not have them.\n");
-#else
-      winip_barf(0);
-#endif
+      fatal("You requested a scan type which requires %s.  Sorry dude.\n", privreq);
     }
     
     if (numdecoys > 0) {
-#ifndef WIN32
-      fatal("Sorry, but you've got to be r00t to use decoys, boy!");
-#else
-      winip_barf(0);
-#endif
+      fatal("Sorry, but decoys (-D) require %s.\n", privreq);
     }
     
     if (fragscan) {
-#ifndef WIN32
-      fatal("Sorry, but fragscan requires r00t privileges\n");
-#else
-      winip_barf(0);
-#endif
+      fatal("Sorry, but fragscan requires %s\n", privreq);
     }
     
     if (osscan) {
-#ifndef WIN32
-      fatal("TCP/IP fingerprinting (for OS scan) requires root privileges which you do not appear to possess.  Sorry, dude.\n");
-#else
-      winip_barf(0);
-#endif
+      fatal("TCP/IP fingerprinting (for OS scan) requires %s.  Sorry, dude.\n", privreq);
     }
   }
   
@@ -465,3 +472,8 @@ void NmapOps::setXSLStyleSheet(char *xslname) {
   if (xsl_stylesheet) free(xsl_stylesheet);
   xsl_stylesheet = xslname? strdup(xslname) : NULL;
 }
+
+void NmapOps::setSpoofMACAddress(u8 *mac_data) {
+  memcpy(spoof_mac, mac_data, 6);
+  spoof_mac_set = true;
+}
