@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h,v 1.22 2004/10/18 16:59:36 fyodor Exp $ */
+/* $Id: NmapOps.h 2839 2005-09-07 08:26:45Z fyodor $ */
 
 class NmapOps {
  public:
@@ -122,11 +122,14 @@ class NmapOps {
   int TimeSinceStartMS(struct timeval *now=NULL); 
   struct in_addr v4source();
   const struct in_addr *v4sourceip();
+
+
+
   bool TCPScan(); /* Returns true if at least one chosen scan type is TCP */
   bool UDPScan(); /* Returns true if at least one chosen scan type is UDP */
 
   /* Returns true if at least one chosen scan type uses raw packets.
-     Itdoes not currently cover cases such as TCP SYN ping scan which
+     It does not currently cover cases such as TCP SYN ping scan which
      can go either way based on whether the user is root or IPv6 is
      being used.  It will return false in those cases where a RawScan
      is not neccessarily used. */
@@ -138,6 +141,26 @@ class NmapOps {
 		             user). */
   int isr00t;
   int debugging;
+
+#define PACKET_SEND_NOPREF 1
+#define PACKET_SEND_ETH_WEAK 2
+#define PACKET_SEND_ETH_STRONG 4
+#define PACKET_SEND_ETH 6
+#define PACKET_SEND_IP_WEAK 8
+#define PACKET_SEND_IP_STRONG 16
+#define PACKET_SEND_IP 24
+
+  /* How should we send raw IP packets?  Nmap can generally use either
+     ethernet or raw ip sockets.  Which is better depends on platform
+     and goals.  A _STRONG preference means that Nmap should use the
+     preferred method whenever it is possible (obviously it isn't
+     always possible -- sending ethernet frames won't work over a PPP
+     connection).  This is useful when the other type doesn't work at
+     all.  A _WEAK preference means that Nmap may use the other type
+     where it is substantially more efficient to do so. For example,
+     Nmap will still do an ARP ping scan of a local network even when
+     the pref is SEND_IP_WEAK */
+  int sendpref;
   bool packetTrace() { return (debugging >= 3)? true : pTrace;  }
   bool versionTrace() { return packetTrace()? true : vTrace;  }
   // Note that packetTrace may turn on at high debug levels even if
@@ -187,6 +210,21 @@ class NmapOps {
   void setMaxTCPScanDelay(unsigned int delayMS) { max_tcp_scan_delay = delayMS; }
   void setMaxUDPScanDelay(unsigned int delayMS) { max_udp_scan_delay = delayMS; }
 
+  /* Sets the Name of the XML stylesheet to be printed in XML output.
+     If this is never called, a default stylesheet distributed with
+     Nmap is used.  If you call it with NULL as the xslname, no
+     stylesheet line is printed. */
+  void setXSLStyleSheet(char *xslname);
+  /* Returns the full path or URL that should be printed in the XML
+     output xml-stylesheet element.  Returns NULL if the whole element
+     should be skipped */
+  char *XSLStyleSheet() { return xsl_stylesheet; }
+
+  /* Sets the spoofed MAC address */
+  void setSpoofMACAddress(u8 *mac_data);
+  /* Gets the spoofed MAC address, but returns NULL if it hasn't been set */
+  const u8 *spoofMACAddress() { return spoof_mac_set? spoof_mac : NULL; }
+
   int max_ips_to_scan; // Used for Random input (-iR) to specify how 
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data_length op */
@@ -206,6 +244,10 @@ class NmapOps {
 			       target_struct_get will eventually set it 
 			       to 0. */
 
+  // Version Detection Options
+  int override_excludeports;
+  int version_intensity;
+
   struct in_addr decoys[MAX_DECOYS];
   int osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
   int osscan_guess;   /* Be more aggressive in guessing OS type */
@@ -217,7 +259,7 @@ class NmapOps {
   int listscan;
   int pingscan;
   int allowall;
-  int fragscan;
+  int fragscan; /* 0 or MTU (without IPv4 header size) */
   int ackscan;
   int bouncescan;
   int connectscan;
@@ -253,5 +295,8 @@ class NmapOps {
   struct timeval start_time;
   bool pTrace; // Whether packet tracing has been enabled
   bool vTrace; // Whether version tracing has been enabled
+  char *xsl_stylesheet;
+  u8 spoof_mac[6];
+  bool spoof_mac_set;
 };
   
