@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOutputTable.cc,v 1.6 2004/08/29 09:12:03 fyodor Exp $ */
+/* $Id: NmapOutputTable.cc 2820 2005-08-24 19:22:11Z fyodor $ */
 
 #include "NmapOutputTable.h"
 #include "utils.h"
@@ -136,7 +136,7 @@ NmapOutputTable::~NmapOutputTable() {
   if (tableout) free(tableout);
 }
 
-void NmapOutputTable::addItem(unsigned int row, unsigned int column, bool copy, char *item, 
+void NmapOutputTable::addItem(unsigned int row, unsigned int column, bool copy, const char *item, 
 			      int itemlen) {
   struct NmapOutputTableCell *cell;
 
@@ -160,7 +160,7 @@ void NmapOutputTable::addItem(unsigned int row, unsigned int column, bool copy,
     memcpy(cell->str, item, itemlen);
     cell->str[itemlen] = '\0';
   } else {
-    cell->str = item;
+    cell->str = (char *) item;
   }
   cell->weAllocated = copy;
 
