@@ -99,7 +99,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc,v 1.13 2004/11/12 09:35:13 fyodor Exp $ */
+/* $Id: TargetGroup.cc 2839 2005-09-07 08:26:45Z fyodor $ */
 
 #include "TargetGroup.h"
 #include "NmapOps.h"
@@ -187,8 +187,8 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     target_net = strtok(hostexp, "/");
     s = strtok(NULL, "");    /* find the end of the token from hostexp */
     netmask  = ( s ) ? atoi(s) : 32;
-    if ((int) netmask < 0 || netmask > 32) {
-      fprintf(stderr, "Illegal netmask value (%d), must be /0 - /32 .  Assuming /32 (one host)\n", netmask);
+    if ((int) netmask <= 0 || netmask > 32) {
+      fprintf(stderr, "Illegal netmask value (%d), must be /1 - /32 .  Assuming /32 (one host)\n", netmask);
       netmask = 32;
     }
     for(i=0; *(hostexp + i); i++) 
@@ -304,9 +304,9 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
  * get_next_host should be used for skipping the last octet :-) 
  * returns: number of hosts skipped */
 int TargetGroup::skip_range(_octet_nums octet) {
-  int hosts_skipped = 0, /* number of hosts skipped */
-      oct = 0,           /* octect number */
-      i;                 /* simple lcv */
+  unsigned long hosts_skipped = 0, /* number of hosts skipped */
+      oct = 0;           /* octect number */
+      int i = 0;                 /* simple lcv */
 
   /* This function is only supported for RANGES! */
   if (targets_type != IPV4_RANGES)
@@ -330,7 +330,7 @@ int TargetGroup::skip_range(_octet_nums octet) {
   }
 
   /* catch if we try to take more than are left */
-  assert(ipsleft >= hosts_skipped - 1);
+  assert(ipsleft + 1>= hosts_skipped);
 
   /* increment the next octect that we can above us */
   for (i = oct; i >= 0; i--) {
@@ -369,7 +369,7 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
   assert(sslen);
 
 
-  if (ipsleft <= 0)
+  if (ipsleft == 0)
     return -1;
   
   if (targets_type == IPV4_NETMASK) {
@@ -441,7 +441,6 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
 #endif // HAVE_IPV6
   }
   ipsleft--;
-  assert(ipsleft >= 0);
   
   /* If we are resuming from a previous scan, we have already finished
      scans up to o.resume_ip.  */
