@@ -9,15 +9,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -27,8 +29,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -55,8 +60,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -82,11 +96,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h,v 1.33 2003/12/02 01:09:39 fyodor Exp $ */
+/* $Id: nbase.h 2443 2004-10-12 09:34:12Z fyodor $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -165,7 +180,7 @@
 #endif
 
 #ifndef HAVE___ATTRIBUTE__
-#define __attribute__(x)
+#define __attribute__(args)
 #endif
 
 #include <stdarg.h>
@@ -278,10 +293,14 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define S_ISDIR(m)      (((m) & _S_IFMT) == _S_IFDIR)
 #endif
 
-#define stat _stat // wtf was ms thinking?
+#define stat _stat /* wtf was ms thinking? */
 #define execve _execve
+
+#if !defined(__GNUC__)
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
+#endif
+
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
 
@@ -301,9 +320,14 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 extern "C" {
 #endif
 
-// Returns the UNIX/Windows errno-equivalent.  Note that the Windows call is socket/networking specific.  The windows error number
-// returned is like WSAMSGSIZE, but nbase.h includes #defines to correlate many of the common UNIX errors with their closest 
-// Windows equivalents.  So you can use EMSGSIZE or EINTR.
+  /* Returns the UNIX/Windows errno-equivalent.  Note that the Windows
+     call is socket/networking specific.  Also, WINDOWS TENDS TO RESET
+     THE ERROR, so it will return success the next time.  So SAVE THE
+     RESULTS and re-use them, don't keep calling socket_errno().  The
+     windows error number returned is like WSAMSGSIZE, but nbase.h
+     includes #defines to correlate many of the common UNIX errors
+     with their closest Windows equivalents.  So you can use EMSGSIZE
+     or EINTR. */
 int socket_errno();
 
 /* The usleep() function is important as well */
