@@ -4,15 +4,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -22,8 +24,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -50,8 +55,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -77,23 +91,26 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_winunix.h,v 1.4 2003/09/16 06:04:55 fyodor Exp $ */
+/* $Id: nbase_winunix.h 2443 2004-10-12 09:34:12Z fyodor $ */
 
 #ifndef NBASE_WINUNIX_H
 #define NBASE_WINUNIX_H
 
 #define _INC_ERRNO  /* supress errno.h */
+#define _ERRNO_H_ /* Also for errno.h suppresion */
 
-//	Supress winsock.h
+/* Supress winsock.h */
 #define _WINSOCKAPI_
+#define WIN32_LEAN_AND_MEAN
 
 #include <windows.h>
 #include <winsock2.h>
-/* #include <ws2tcpip.h> // IPv6 stuff */
+#include <ws2tcpip.h> /* IPv6 stuff */
 #include <time.h>
 #include <iptypes.h>
 #include <stdlib.h>
@@ -103,39 +120,40 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <process.h>
-//#include <errno.h>
 #include <limits.h>
 #include <WINCRYPT.H>
 #include <math.h>
-//#include <packet_types.h>
 
+#ifndef HAVE_U_INT8_T
+#define HAVE_U_INT8_T
 typedef unsigned char u_int8_t;
+#endif
+
+#ifndef HAVE_U_INT16_T
+#define HAVE_U_INT16_T
 typedef unsigned short u_int16_t;
+#endif
 
+#ifndef HAVE_U_INT32_T
+#define HAVE_U_INT32_T
 typedef unsigned long u_int32_t;
-typedef unsigned int ssize_t;
+#endif
 
-//#define HAVE_STRUCT_IP
-//#define HAVE_STRUCT_ICMP
+#ifndef _SSIZE_T_
+typedef unsigned int ssize_t;
+#endif
 
 #define SIOCGIFCONF     0x8912          /* get iface list */
 
 #ifndef GLOBALS
 #define GLOBALS 1
 
-//extern struct interface_info global_adapter;
 #endif
 
 /* Disables VC++ warning:
   "integral size mismatch in argument; conversion supplied".  Perhaps
   I should try to fix this with casts at some point */
-// #pragma warning(disable: 4761)
-
-/* #define signal(x,y) ((void)0)	// ignore for now
-                                // later release may set console handlers
-*/
-
-extern struct winops wo;
+/* #pragma warning(disable: 4761) */
 
 #define munmap(ptr, len) win32_munmap(ptr, len)
 int nmapwin_isroot();
@@ -144,6 +162,7 @@ int nmapwin_isroot();
 #define ECONNABORTED    WSAECONNABORTED
 #define ECONNRESET      WSAECONNRESET
 #define ECONNREFUSED    WSAECONNREFUSED
+#undef  EAGAIN
 #define EAGAIN		WSAEWOULDBLOCK
 #define EHOSTUNREACH	WSAEHOSTUNREACH
 #define ENETDOWN	WSAENETDOWN
@@ -152,26 +171,32 @@ int nmapwin_isroot();
 #define ETIMEDOUT	WSAETIMEDOUT
 #define EHOSTDOWN	WSAEHOSTDOWN
 #define EINPROGRESS	WSAEINPROGRESS
+#undef  EINVAL
 #define EINVAL          WSAEINVAL      /* Invalid argument */
+#undef  EPERM
 #define EPERM           WSAEACCES      /* Operation not permitted */
+#undef  EACCES
+#define EACCES          WSAEACCES     /* Operation not permitted */
+#undef  EINTR
 #define EINTR           WSAEINTR      /* Interrupted system call */
 #define ENOBUFS         WSAENOBUFS     /* No buffer space available */
+#undef  ENOENT
 #define ENOENT          WSAENOENT      /* No such file or directory */
 #define EMSGSIZE        WSAEMSGSIZE    /* Message too long */
+#undef  ENOMEM
 #define ENOMEM          WSAENOBUFS
+#undef  EIO
 #define EIO             WSASYSCALLFAILURE
+
 #define close(x) my_close(x)
 /* #define read(x,y,z) recv(x,(char*)(y),z,0) */
-int my_close(int sd);
+
+#ifdef __cplusplus
+  extern "C" int my_close(int sd);
+#else
+  int my_close(int sd);
+#endif
 
 typedef unsigned short u_short_t;
 
-int win32_sendto(int sd, const char *packet, int len, 
-	   unsigned int flags, struct sockaddr *to, int tolen);
-
-int win32_socket(int af, int type, int proto);
-
-#define socket(af, type, proto) win32_socket(af, type, proto)
-#define sendto(sd, packet, len, flags, to, tolen) win32_sendto(sd, packet, len, flags, to, tolen)
-
 #endif /* NBASE_WINUNIX_H */
