@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc,v 1.60 2005/02/05 22:37:55 fyodor Exp $ */
+/* $Id: nmap.cc 2868 2005-09-12 08:46:04Z fyodor $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -106,6 +106,12 @@
 #include "idle_scan.h"
 #include "timing.h"
 #include "NmapOps.h"
+#include "MACLookup.h"
+#ifdef WIN32
+#include "winfix.h"
+#endif
+
+using namespace std;
 
 /* global options */
 extern char *optarg;
@@ -223,17 +229,18 @@ int nmap_main(int argc, char *argv[]) {
   Target *currenths;
   vector<Target *> Targets;
   char *proberr;
-  bool skip_winip_init = false;
   char emptystring[1];
   int sourceaddrwarning = 0; /* Have we warned them yet about unguessable
 				source addresses? */
   unsigned int ideal_scan_group_sz = 0;
   char hostname[MAXHOSTNAMELEN + 1] = "";
+  const char *spoofmac = NULL;
   time_t timep;
   char mytime[128];
   struct sockaddr_storage ss;
   size_t sslen;
   int option_index;
+  bool iflist = false;
   struct option long_options[] =
     {
       {"version", no_argument, 0, 'V'},
@@ -241,9 +248,11 @@ int nmap_main(int argc, char *argv[]) {
       {"datadir", required_argument, 0, 0},
       {"debug", optional_argument, 0, 'd'},
       {"help", no_argument, 0, 'h'},
+      {"iflist", no_argument, 0, 0},
       {"max_parallelism", required_argument, 0, 'M'},
       {"min_parallelism", required_argument, 0, 0},
       {"timing", required_argument, 0, 'T'},
+      {"timing", no_argument, 0, 0},
       {"max_rtt_timeout", required_argument, 0, 0},
       {"min_rtt_timeout", required_argument, 0, 0},
       {"initial_rtt_timeout", required_argument, 0, 0},
@@ -273,26 +282,23 @@ int nmap_main(int argc, char *argv[]) {
       {"version_trace", no_argument, 0, 0}, /* Display -sV related activity */
       {"fuzzy", no_argument, 0, 0}, /* Alias for osscan_guess */
       {"data_length", required_argument, 0, 0},
+      {"send_eth", no_argument, 0, 0},
+      {"send_ip", no_argument, 0, 0},
       {"stylesheet", required_argument, 0, 0},
-      {"no-stylesheet", no_argument, 0, 0},
+      {"no_stylesheet", no_argument, 0, 0},
       {"rH", no_argument, 0, 0},
       {"vv", no_argument, 0, 0},
       {"ff", no_argument, 0, 0},
+      {"privileged", no_argument, 0, 0},
       {"mtu", required_argument, 0, 0},
       {"append_output", no_argument, 0, 0},
       {"noninteractive", no_argument, 0, 0},
+      {"spoof_mac", required_argument, 0, 0},
       {"ttl", required_argument, 0, 0}, /* Time to live */
-#ifdef WIN32
-      {"win_list_interfaces", no_argument, 0, 0},
-      {"win_norawsock", no_argument, 0, 0}, 
-      {"win_forcerawsock", no_argument, 0, 0}, 
-      {"win_nopcap", no_argument, 0, 0}, 
-      {"win_nt4route", no_argument, 0, 0}, 
-      {"win_noiphlpapi", no_argument, 0, 0}, 
-      {"win_help", no_argument, 0, 0},
-      {"win_trace", no_argument, 0, 0},
-	  {"win_skip_winip_init", no_argument, 0, 0},
-#endif
+      {"allports", no_argument, 0, 0},
+      {"version_intensity", required_argument, 0, 0},
+      {"version_light", no_argument, 0, 0},
+      {"version_all", no_argument, 0, 0},
       {0, 0, 0, 0}
     };
 
@@ -351,6 +357,8 @@ int nmap_main(int argc, char *argv[]) {
 	if (o.scanflags < 0) {
 	  fatal("--scanflags option must be a number between 0 and 255 (inclusive) or a string like \"URGPSHFIN\".");
 	}
+      } else if (strcmp(long_options[option_index].name, "iflist") == 0 ) {
+	iflist = true;
       } else if (strcmp(long_options[option_index].name, "min_parallelism") == 0 ) {
 	o.min_parallelism = atoi(optarg); 
 	if (o.min_parallelism < 1) fatal("Argument to --min_parallelism must be at least 1!");
@@ -362,6 +370,9 @@ int nmap_main(int argc, char *argv[]) {
 	if (o.host_timeout <= 200) {
 	  fatal("host_timeout is given in milliseconds and must be greater than 200");
 	}
+	if (o.host_timeout <= 1000) {
+	  error("host_timeout is given in milliseconds, so you specified less than a second (%lims). This is allowed but not recommended.", o.host_timeout);
+	}
       } else if (strcmp(long_options[option_index].name, "ttl") == 0) {
 	o.ttl = atoi(optarg);
 	if (o.ttl < 0 || o.ttl > 255) {
@@ -369,38 +380,24 @@ int nmap_main(int argc, char *argv[]) {
 	}
       } else if (strcmp(long_options[option_index].name, "datadir") == 0) {
 	o.datadir = strdup(optarg);
-#ifdef WIN32
-      } else if (strcmp(long_options[option_index].name, "win_list_interfaces") == 0 ) { 
-	wo.listinterfaces = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_norawsock") == 0 ) { 
-	wo.norawsock = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_forcerawsock") == 0 ) { 
-	wo.forcerawsock = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_nopcap") == 0 ) { 
-	wo.nopcap = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_nt4route") == 0 ) { 
-	wo.nt4route = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_noiphlpapi") == 0 ) { 
-	wo.noiphlpapi = 1; 
-      } else if (strcmp(long_options[option_index].name, "win_trace") == 0 ) { 
-	wo.trace++; 
-      } else if (strcmp(long_options[option_index].name, "win_skip_winip_init") == 0 ) { 
-	skip_winip_init = true;
-      } else if (strcmp(long_options[option_index].name, "win_help") == 0 ) { 
-	printf("Windows-specific options:\n\n"); 
-	printf(" --win_list_interfaces : list all network interfaces\n"); 
-	printf(" --win_norawsock       : disable raw socket support\n"); 
-	printf(" --win_forcerawsock    : try raw sockets even on non-W2K systems\n"); 
-	printf(" --win_nopcap          : disable winpcap support\n"); 
-	printf(" --win_nt4route        : test nt4 route code\n"); 
-	printf(" --win_noiphlpapi      : test response to lack of iphlpapi.dll\n"); 
-	printf(" --win_trace           : trace through raw IP initialization\n");
-	exit(0);
-#endif
       } else if (strcmp(long_options[option_index].name, "append_output") == 0) {
 	o.append_output = 1;
       } else if (strcmp(long_options[option_index].name, "noninteractive") == 0) {
 	/* Do nothing */
+      } else if (strcmp(long_options[option_index].name, "spoof_mac") == 0) {
+	/* I need to deal with this later, once I'm sure that I have output
+	   files set up, --datadir, etc. */
+	spoofmac = optarg;
+      } else if (strcmp(long_options[option_index].name, "allports") == 0) {
+        o.override_excludeports = 1;
+      } else if (strcmp(long_options[option_index].name, "version_intensity") == 0) {
+	o.version_intensity = atoi(optarg);
+	if (o.version_intensity < 0 || o.version_intensity > 9)
+		fatal("version_intensity must be between 0 and 9");
+      } else if (strcmp(long_options[option_index].name, "version_light") == 0) {
+        o.version_intensity = 2;
+      } else if (strcmp(long_options[option_index].name, "version_all") == 0) {
+        o.version_intensity = 9;
       } else if (strcmp(long_options[option_index].name, "scan_delay") == 0) {
 	o.scan_delay = atoi(optarg);
 	if (o.scan_delay <= 0) {
@@ -438,9 +435,13 @@ int nmap_main(int argc, char *argv[]) {
 	  o.extra_payload = (char *) safe_malloc(o.extra_payload_length);
 	  get_random_bytes(o.extra_payload, o.extra_payload_length);
 	}
+      } else if (strcmp(long_options[option_index].name, "send_eth") == 0) {
+	o.sendpref = PACKET_SEND_ETH_STRONG;
+      } else if (strcmp(long_options[option_index].name, "send_ip") == 0) {
+	o.sendpref = PACKET_SEND_IP_STRONG;
       } else if (strcmp(long_options[option_index].name, "stylesheet") == 0) {
 	o.setXSLStyleSheet(optarg);
-      } else if (strcmp(long_options[option_index].name, "no-stylesheet") == 0) {
+      } else if (strcmp(long_options[option_index].name, "no_stylesheet") == 0) {
 	o.setXSLStyleSheet(NULL);
       } else if (strcmp(long_options[option_index].name, "oN") == 0) {
 	normalfilename = optarg;
@@ -487,8 +488,9 @@ int nmap_main(int argc, char *argv[]) {
 	/* Compatability hack ... ugly */
 	o.verbose += 2;
       } else if (strcmp(long_options[option_index].name, "ff") == 0) {
-	/* Compatability hack ... ugly */
-	o.fragscan = 8;
+	o.fragscan += 16; 
+      } else if (strcmp(long_options[option_index].name, "privileged") == 0) {
+	o.isr00t = 1;
       } else if (strcmp(long_options[option_index].name, "mtu") == 0) {
         o.fragscan = atoi(optarg);
         if (o.fragscan <= 0 || o.fragscan % 8 != 0)
@@ -600,6 +602,8 @@ int nmap_main(int argc, char *argv[]) {
 	o.pingtype |= PINGTYPE_ICMP_TS;
       else if (*optarg == '0' || *optarg == 'N' || *optarg == 'D')      
 	o.pingtype = PINGTYPE_NONE;
+      else if (*optarg == 'R')
+	o.pingtype |= PINGTYPE_ARP;
       else if (*optarg == 'S') {
 	o.pingtype |= (PINGTYPE_TCP|PINGTYPE_TCP_USE_SYN);
 	if (isdigit((int) *(optarg+1)))
@@ -752,8 +756,7 @@ int nmap_main(int argc, char *argv[]) {
   }
 
 #ifdef WIN32
-  if (!skip_winip_init)
-    winip_postopt_init();
+    win_init();
 #endif
 
 #if HAVE_SIGNAL
@@ -790,7 +793,10 @@ int nmap_main(int argc, char *argv[]) {
     if (o.verbose && tm->tm_mon == 8 && tm->tm_mday == 1) {
       log_write(LOG_STDOUT|LOG_SKID, "Happy %dth Birthday to Nmap, may it live to be %d!\n", tm->tm_year - 97, tm->tm_year + 3 );
     }
-
+    if (iflist) {
+      print_iflist();
+      exit(0);
+    }
   }
 
   if ((o.pingscan || o.listscan) && fastscan) {
@@ -809,6 +815,59 @@ int nmap_main(int argc, char *argv[]) {
     fatal("You cannot use -F (fast scan) or -p (explicit port selection) with PING scan or LIST scan");
   }
 
+#ifdef WIN32
+  if (o.sendpref & PACKET_SEND_IP) {
+	  error("WARNING: raw IP (rather than raw ethernet) packet sending attempted on Windows. This probably won't work.  Consider --send_eth next time.\n");
+
+  }
+#endif
+  if (spoofmac) {
+    u8 mac_data[6];
+    int pos = 0; /* Next index of mac_data to fill in */
+    char tmphex[3];
+    /* A zero means set it all randomly.  Anything that is all digits
+       or colons is treated as a prefix, with remaining characters for
+       the 6-byte MAC (if any) chosen randomly.  Otherwise, it is
+       treated as a vendor string for lookup in nmap-mac-prefixes */
+    if (strcmp(spoofmac, "0") == 0) {
+      pos = 0;
+    } else {
+      const char *p = spoofmac;
+      while(*p) { 
+	if (*p == ':') p++;
+	if (isxdigit(*p) && isxdigit(*(p+1))) {
+	  if (pos >= 6) fatal("Bogus --spoof_mac value encountered (%s) -- only up to 6 bytes permitted", spoofmac);
+	  tmphex[0] = *p; tmphex[1] = *(p+1); tmphex[2] = '\0';
+	  mac_data[pos] = (u8) strtol(tmphex, NULL, 16);
+	  pos++;
+	  p += 2;
+	} else break;
+      }
+      if (*p) {
+	/* Failed to parse it as a MAC prefix -- treating as a vendor substring instead */
+	if (!MACCorp2Prefix(spoofmac, mac_data))
+	  fatal("Could not parse as a prefix nor find as a vendor substring the given --spoof_mac argument: %s.  If you are giving hex digits, there must be an even number of them.", spoofmac);
+	pos = 3;
+      }
+    }
+    if (pos < 6) {
+      get_random_bytes(mac_data + pos, 6 - pos);
+    }
+    /* Got the new MAC! */
+    const char *vend = MACPrefix2Corp(mac_data);
+    log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, 
+	      "Spoofing MAC address %02X:%02X:%02X:%02X:%02X:%02X (%s)\n",
+	      mac_data[0], mac_data[1], mac_data[2], mac_data[3], mac_data[4],
+	      mac_data[5], vend? vend : "No registered vendor");
+    o.setSpoofMACAddress(mac_data);
+
+    /* If they want to spoof the MAC address, we should at least make
+       some effort to actually send raw ethernet frames rather than IP
+       packets (which would use the real IP */
+    if (o.sendpref != PACKET_SEND_IP_STRONG)
+      o.sendpref = PACKET_SEND_ETH_STRONG;
+  }
+
   if (!ports) {
     if (o.ipprotscan) {
       ports = getdefaultprots();
@@ -1076,16 +1135,16 @@ int nmap_main(int argc, char *argv[]) {
 	    }
 	  }
 	}
-	if (!*currenths->device)
-	  if (ipaddr2devname( currenths->device, currenths->v4sourceip()) != 0)
-	    fatal("Could not figure out what device to send the packet out on!  You might possibly want to try -S (but this is probably a bigger problem).  If you are trying to sp00f the source of a SYN/FIN scan with -S <fakeip>, then you must use -e eth0 (or other devicename) to tell us what interface to use.\n");
+	if (!currenths->deviceName())
+	  fatal("Do not have appropriate device name for target");
 	
 	/* Groups should generally use the same device as properties
 	   change quite a bit between devices.  Plus dealing with a
 	   multi-device group can be a pain programmatically. So if
 	   this Target has a different device the rest, we give it
 	   back. */
-	if (Targets.size() > 0 && strcmp(Targets[Targets.size() - 1]->device, currenths->device)) {
+	if (Targets.size() > 0 && 
+	    strcmp(Targets[Targets.size() - 1]->deviceName(), currenths->deviceName())) {
 	  returnhost(hstate);
 	  numhosts_scanned--; numhosts_up--;
 	  break;
@@ -1104,7 +1163,6 @@ int nmap_main(int argc, char *argv[]) {
       o.decoys[o.decoyturn] = Targets[0]->v4source();
     
     /* I now have the group for scanning in the Targets vector */
-    /* TODO: Add parallel-capable scans here */
     if (o.synscan)
       ultra_scan(Targets, ports, SYN_SCAN);
     
@@ -1162,7 +1220,7 @@ int nmap_main(int argc, char *argv[]) {
       // Should be host parallelized.  Though rarely takes a huge amt. of time.
       if (o.osscan) 
 	os_scan(currenths);
-      
+
     /* Now I can do the output and such for each host */
       log_write(LOG_XML, "<host>");
       write_host_status(currenths, resolve_all);
@@ -1175,6 +1233,7 @@ int nmap_main(int argc, char *argv[]) {
 	printportoutput(currenths, &currenths->ports);
 	printmacinfo(currenths);
 	printosscanoutput(currenths);
+	printserviceinfooutput(currenths);
       }
     
       if (o.debugging) 
@@ -1499,11 +1558,6 @@ struct scan_lists *getpts(char *origexpr) {
 }
 
 void printusage(char *name, int rc) {
-#ifdef WIN32
-#define WIN32_PRINTF "  --win_help Windows-specific features\n"
-#else
-#define WIN32_PRINTF
-#endif
   printf(
 	 "Nmap %s Usage: nmap [Scan Type(s)] [Options] <host or net list>\n"
 	 "Some Common Scan Types ('*' options require root privileges)\n"
@@ -1528,7 +1582,6 @@ void printusage(char *name, int rc) {
 	 "  -iL <inputfile> Get targets from file; Use '-' for stdin\n"
 	 "* -S <your_IP>/-e <devicename> Specify source address or network interface\n"
 	 "  --interactive Go into interactive mode (then press h for help)\n"
-         WIN32_PRINTF
 	 "Example: nmap -v -sS -O www.my.com 192.168.0.0/16 '192.88-90.*.*'\n"
 	 "SEE THE MAN PAGE FOR MANY MORE OPTIONS, DESCRIPTIONS, AND EXAMPLES \n", NMAP_VERSION);
   exit(rc);
@@ -1662,7 +1715,9 @@ char *tsseqclass2ascii(int seqclass) {
  * 2001 (www.junk.org is an example of a new address in this range).
  *
  * Check <http://www.iana.org/assignments/ipv4-address-space> for
- * the most recent assigments.
+ * the most recent assigments and
+ * <http://www.cymru.com/Documents/bogon-bn-nonagg.txt> for bogon
+ * netblocks.
  */
 
 int ip_is_reserved(struct in_addr *ip)
@@ -1670,24 +1725,6 @@ int ip_is_reserved(struct in_addr *ip)
   char *ipc = (char *) &(ip->s_addr);
   unsigned char i1 = ipc[0], i2 = ipc[1], i3 = ipc[2], i4 = ipc[3];
 
-  /* 224-239/8 is all multicast stuff */
-  /* 240-255/8 is IANA reserved */
-  if (i1 >= 224)
-    return 1;
-
-  /* 096-126/8 is IANA reserved */
-  /* 127/8 is reserved for loopback */
-  if (i1 >= 96 && i1 <= 127)
-    return 1;
-
-  /* 070-079/8 is IANA reserved */
-  if (i1 >= 70 && i1 <= 79)
-    return 1;
-
-  /* 083-095/8 is IANA reserved */
-  if (i1 >= 83 && i1 <= 95)
-    return 1;
-
   /* do all the /7's and /8's with a big switch statement, hopefully the
    * compiler will be able to optimize this a little better using a jump table
    * or what have you
@@ -1707,29 +1744,50 @@ int ip_is_reserved(struct in_addr *ip)
     case 36:        /* 036/8 is IANA reserved       */
     case 37:        /* 037/8 is IANA reserved       */
     case 39:        /* 039/8 is IANA reserved       */
-    case 41:        /* 041/8 is IANA reserved       */
     case 42:        /* 042/8 is IANA reserved       */
+    case 49:        /* 049/8 is IANA reserved       */
+    case 50:        /* 050/8 is IANA reserved       */
     case 55:        /* misc. U.S.A. Armed forces    */
-    case 58:        /* 058/8 is IANA reserved       */
-    case 59:        /* 059/8 is IANA reserved       */
-    case 60:        /* 060/8 is IANA reserved       */
-    case 197:
+    case 127:       /* 127/8 is reserved for loopback */
+    case 197:       /* 197/8 is IANA reserved       */
+    case 223:       /* 223/8 is IANA reserved       */
       return 1;
     default:
       break;
     }
 
+
+  /* 077-079/8 is IANA reserved */
+  if (i1 >= 77 && i1 <= 79)
+    return 1;
+
+  /* 092-123/8 is IANA reserved */
+  if (i1 >= 92 && i1 <= 123)
+    return 1;
+
   /* 172.16.0.0/12 is reserved for private nets by RFC1819 */
   if (i1 == 172 && i2 >= 16 && i2 <= 31)
     return 1;
 
+  /* 173-187/8 is IANA reserved */
+  if (i1 >= 173 && i1 <= 187)
+    return 1;
+
   /* 192.168.0.0/16 is reserved for private nets by RFC1819 */
   /* 192.0.2.0/24 is reserved for documentation and examples */
+  /* 192.88.99.0/24 is used as 6to4 Relay anycast prefix by RFC3068 */
   if (i1 == 192) {
     if (i2 == 168)
       return 1;
-    else if (i2 == 0 && i3 == 2)
+    if (i2 == 0 && i3 == 2)
       return 1;
+    if (i2 == 88 && i3 == 99)
+      return 1;
+  }
+
+  /* 198.18.0.0/15 is used for benchmark tests by RFC2544 */
+  if (i1 == 198 && i2 == 18 && i3 >= 1 && i3 <= 64) {
+    return 1;
   }
 
   /* reserved for DHCP clients seeking addresses, not routable outside LAN */
@@ -1741,6 +1799,11 @@ int ip_is_reserved(struct in_addr *ip)
   if (i1 == 204 && i2 == 152 && (i3 == 64 || i3 == 65))
     return 1;
 
+  /* 224-239/8 is all multicast stuff */
+  /* 240-255/8 is IANA reserved */
+  if (i1 >= 224)
+    return 1;
+
   /* 255.255.255.255, note we already tested for i1 in this range */
   if (i2 == 255 && i3 == 255 && i4 == 255)
     return 1;
@@ -1794,6 +1857,8 @@ char *scantype2str(stype scantype) {
   case RPC_SCAN: return "RPCGrind Scan"; break;
   case MAIMON_SCAN: return "Maimon Scan"; break;
   case IPPROT_SCAN: return "IPProto Scan"; break;
+  case PING_SCAN: return "Ping Scan"; break;
+  case PING_SCAN_ARP: return "ARP Ping Scan"; break;
   default: assert(0); break;
   }
 
