@@ -6,15 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,8 +26,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -52,8 +57,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -79,11 +93,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.h,v 1.100 2003/09/11 02:12:57 fyodor Exp $ */
+/* $Id: nmap.h 2839 2005-09-07 08:26:45Z fyodor $ */
 
 #ifndef NMAP_H
 #define NMAP_H
@@ -203,15 +218,15 @@ void *realloc();
 #include <pwd.h>
 #endif
 
-#ifndef NETINET_IN_SYSTEM_H  /* why the HELL does OpenBSD not do this? */
+#ifndef NETINET_IN_SYSTEM_H  /* why does OpenBSD not do this? */
 #include <netinet/in_systm.h> /* defines n_long needed for netinet/ip.h */
 #define NETINET_IN_SYSTEM_H
 #endif
-#ifndef NETINET_IP_H  /* why the HELL does OpenBSD not do this? */
+#ifndef NETINET_IP_H  /* why does OpenBSD not do this? */
 #include <netinet/ip.h> 
 #define NETINET_IP_H
 #endif
-#include <netinet/ip_icmp.h> 
+// #include <netinet/ip_icmp.h> 
 
 #if HAVE_ARPA_INET_H
 #include <arpa/inet.h>
@@ -222,7 +237,7 @@ void *realloc();
 #ifndef __FAVOR_BSD
 #define __FAVOR_BSD
 #endif
-#ifndef NETINET_TCP_H  /* why the HELL does OpenBSD not do this? */
+#ifndef NETINET_TCP_H  /* why does OpenBSD not do this? */
 #include <netinet/tcp.h>          /*#include <netinet/ip_tcp.h>*/
 #define NETINET_TCP_H
 #endif
@@ -232,16 +247,18 @@ void *realloc();
 #endif
 
 /*#include <net/if_arp.h> *//* defines struct arphdr needed for if_ether.h */
-#ifndef NET_IF_H  /* why the HELL does OpenBSD not do this? */
-#include <net/if.h>
-#define NET_IF_H
-#endif
-#if HAVE_NETINET_IF_ETHER_H 
-#ifndef NETINET_IF_ETHER_H
-#include <netinet/if_ether.h>
-#define NETINET_IF_ETHER_H
-#endif /* NETINET_IF_ETHER_H */
-#endif /* HAVE_NETINET_IF_ETHER_H */
+// #if HAVE_NET_IF_H
+// #ifndef NET_IF_H  /* why doesn't OpenBSD do this?! */
+// #include <net/if.h>
+// #define NET_IF_H
+// #endif
+// #endif
+// #if HAVE_NETINET_IF_ETHER_H 
+// #ifndef NETINET_IF_ETHER_H
+// #include <netinet/if_ether.h>
+// #define NETINET_IF_ETHER_H
+// #endif /* NETINET_IF_ETHER_H */
+// #endif /* HAVE_NETINET_IF_ETHER_H */
 
 /*******  DEFINES  ************/
 
@@ -287,16 +304,35 @@ void *realloc();
 
 #define MAX_DECOYS 128 /* How many decoys are allowed? */
 
+#define MAXFALLBACKS 20 /* How many comma separated fallbacks are allowed in the service-probes file? */
+
+/* Default maximum send delay between probes to the same host */
+#ifndef MAX_TCP_SCAN_DELAY
+#define MAX_TCP_SCAN_DELAY 1000
+#endif
+
+#ifndef MAX_UDP_SCAN_DELAY
+#define MAX_UDP_SCAN_DELAY 1000
+#endif
+
+/* Maximum number of extra hostnames, OSs, and devices, we
+   consider when outputing the extra service info fields */
+#define MAX_SERVICE_INFO_FIELDS 5
+
+/* We wait at least 100 ms for a response by default - while that
+   seems aggressive, waiting too long can cause us to fail to detect
+   drops until many probes later on extremely low-latency
+   networks (such as localhost scans).  */
+#ifndef MIN_RTT_TIMEOUT
+#define MIN_RTT_TIMEOUT 100 
+#endif
+
 #ifndef MAX_RTT_TIMEOUT
 #define MAX_RTT_TIMEOUT 10000 /* Never allow more than 10 secs for packet round
 				 trip */
 #endif
 
-#ifndef MIN_RTT_TIMEOUT
-#define MIN_RTT_TIMEOUT 300 /* We will always wait at least 300 ms for a response */
-#endif
-
-#define INITIAL_RTT_TIMEOUT 6000 /* Allow 6 seconds at first for packet responses */
+#define INITIAL_RTT_TIMEOUT 1000 /* Allow 1 second initially for packet responses */
 #define HOST_TIMEOUT    0 /* By default allow unlimited time to scan each host */
 
 /* If nmap is called with one of the names below, it will start up in interactive mode -- alternatively, you can rename Nmap any of the following names to have it start up interactivey by default.  */
@@ -305,7 +341,7 @@ void *realloc();
 /* Number of hosts we pre-ping and then scan.  We do a lot more if
    randomize_hosts is set.  Every one you add to this leads to ~1K of
    extra always-resident memory in nmap */
-#define HOST_GROUP_SZ 1024
+#define PING_GROUP_SZ 2048
 
 /* DO NOT change stuff after this point */
 #define UC(b)   (((int)b)&0xff)
@@ -327,7 +363,9 @@ void *realloc();
 #define PINGTYPE_RAWTCP 128
 #define PINGTYPE_CONNECTTCP 256
 #define PINGTYPE_UDP  512
+#define PINGTYPE_ARP 1024
 
+#define DEFAULT_PING_TYPES PINGTYPE_TCP|PINGTYPE_TCP_USE_ACK|PINGTYPE_ICMP_PING
 /* TCP/IP ISN sequence prediction classes */
 #define SEQ_UNKNOWN 0
 #define SEQ_64K 1
@@ -358,16 +396,6 @@ void *realloc();
 #define MAXHOSTNAMELEN 64
 #endif
 
-#ifndef BSDFIX
-#if FREEBSD || BSDI || NETBSD
-#define BSDFIX(x) x
-#define BSDUFIX(x) x
-#else
-#define BSDFIX(x) htons(x)
-#define BSDUFIX(x) ntohs(x)
-#endif
-#endif /* BSDFIX */
-
 #ifndef recvfrom6_t
 #  define recvfrom6_t int
 #endif
@@ -399,8 +427,6 @@ void printusage(char *name, int rc);
 /* print Interactive usage information */
 void printinteractiveusage();
 
-/* Scan helper functions */
-unsigned long calculate_sleep(struct in_addr target);
 int check_ident_port(struct in_addr target);
 
 int ftp_anon_connect(struct ftpinfo *ftp);
@@ -413,7 +439,6 @@ int getidentinfoz(struct in_addr target, u16 localport, u16 remoteport,
 
 /* socket manipulation functions */
 void init_socket(int sd);
-int recvtime(int sd, char *buf, int len, int seconds);
 
 /* RAW packet building/dissasembling stuff */
 int isup(struct in_addr target);
@@ -442,7 +467,6 @@ char *tsseqclass2ascii(int seqclass);
 const char *seqidx2difficultystr(unsigned long idx);
 int nmap_fetchfile(char *filename_returned, int bufferlen, char *file);
 int fileexistsandisreadable(char *pathname);
-int check_firewallmode(Target *target, struct scanstats *ss);
 int gather_logfile_resumption_state(char *fname, int *myargc, char ***myargv);
 
 /* From glibc 2.0.6 because Solaris doesn't seem to have this function */
