@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.h,v 1.16 2004/10/12 09:34:11 fyodor Exp $ */
+/* $Id: output.h 2820 2005-08-24 19:22:11Z fyodor $ */
 
 #ifndef OUTPUT_H
 #define OUTPUT_H
@@ -191,6 +191,14 @@ void write_host_status(Target *currenths, int resolve_all);
    if an OS Scan was performed */
 void printosscanoutput(Target *currenths);
 
+/* Prints the alternate hostname/OS/device information we got from the
+   service scan (if it was performed) */
+void printserviceinfooutput(Target *currenths);
+
+/* Print a detailed list of Nmap interfaces and routes to
+   normal/skiddy/stdout output */
+int print_iflist(void);
+
 /* Prints the statistics and other information that goes at the very end
    of an Nmap run */
 void printfinaloutput(int numhosts_scanned, int numhosts_up, 
